import streamlit as st
from streamlit.components.v1 import html
import matplotlib
import requests
from io import BytesIO


try:
    from imageio.v2 import imread
except:
    from imageio import imread
import matplotlib.pyplot as plt
from stardist import random_label_cmap
import numpy as np

from myoquant.common_func import (
    load_cellpose,
    load_stardist,
    run_cellpose,
    run_stardist,
    is_gpu_availiable,
    df_from_cellpose_mask,
    df_from_stardist_mask,
)
from myoquant.HE_analysis import (
    predict_all_cells,
    extract_ROIs,
    single_cell_analysis,
    paint_histo_img,
)


use_GPU = is_gpu_availiable()


@st.cache_resource
def st_load_cellpose():
    return load_cellpose()


@st.cache_resource
def st_load_stardist(fluo=False):
    return load_stardist(fluo)


@st.cache_data
def st_run_cellpose(image_ndarray, _model):
    return run_cellpose(image_ndarray, _model)


@st.cache_data
def st_run_stardist(image_ndarray, _model, nms_thresh, prob_thresh):
    return run_stardist(image_ndarray, _model, nms_thresh, prob_thresh)


@st.cache_data
def st_df_from_cellpose_mask(mask):
    return df_from_cellpose_mask(mask)


@st.cache_data
def st_df_from_stardist_mask(mask):
    return df_from_stardist_mask(mask)


@st.cache_data
def st_predict_all_cells(
    image_ndarray, df_cellpose, mask_stardist, internalised_threshold
):
    return predict_all_cells(
        image_ndarray, df_cellpose, mask_stardist, internalised_threshold
    )


@st.cache_data
def st_extract_ROIs(image_ndarray, selected_fiber, df_cellpose, mask_stardist):
    return extract_ROIs(image_ndarray, selected_fiber, df_cellpose, mask_stardist)


@st.cache_data
def st_single_cell_analysis(
    single_cell_img,
    single_cell_mask,
    df_nuc_single,
    x_fiber,
    y_fiber,
    selected_fiber,
    internalised_threshold,
    draw_and_return=True,
):
    return single_cell_analysis(
        single_cell_img,
        single_cell_mask,
        df_nuc_single,
        x_fiber,
        y_fiber,
        selected_fiber + 1,
        internalised_threshold,
        draw_and_return=True,
    )


@st.cache_data
def st_paint_histo_img(image_ndarray, df_cellpose, cellpose_df_stat):
    return paint_histo_img(image_ndarray, df_cellpose, cellpose_df_stat)


with st.sidebar:
    st.write("Nuclei detection Parameters (Stardist)")
    nms_thresh = st.slider("Stardist NMS Tresh", 0.0, 1.0, 0.4, 0.1)
    prob_thresh = st.slider("Stardist Prob Tresh", 0.5, 1.0, 0.5, 0.05)
    st.write("Nuclei Classification Parameter")
    eccentricity_thresh = st.slider("Eccentricity Score Tresh", 0.0, 1.0, 0.75, 0.05)

model_cellpose = st_load_cellpose()
model_stardist = st_load_stardist(fluo=True)

st.title("Breast Muscle Fiber Analysis")
st.write(
    "This demo will automatically detect cells and nucleus in the image and try to quantify a certain number of features."
)


default_file_url_3 = "https://raw.githubusercontent.com/lambda-science/MyoQuant/refs/heads/main/sample_img/cytoplasmes.tif"
default_file_url_4 = "https://raw.githubusercontent.com/lambda-science/MyoQuant/refs/heads/main/sample_img/noyaux.tif"

st.write(
    "Upload your singles channels images (cytoplasme images and nuclei image) OR click the Load Default File button !"
)
col1, col2 = st.columns(2)
with col1:
    uploaded_file_cyto = st.file_uploader("Choose a file (cyto)")
    uploaded_file_nuc = st.file_uploader("Choose a file (nuc)")
    if uploaded_file_cyto is not None:
        st.session_state["uploaded_file3"] = uploaded_file_cyto
    if uploaded_file_nuc is not None:
        st.session_state["uploaded_file4"] = uploaded_file_nuc

with col2:
    if st.button("Load Default Files", type="primary"):
        # Download the default file
        response3 = requests.get(default_file_url_3)
        response4 = requests.get(default_file_url_4)
        # Convert the downloaded content into a file-like object
        uploaded_file_cyto = BytesIO(response3.content)
        uploaded_file_nuc = BytesIO(response4.content)

        st.session_state["uploaded_file3"] = uploaded_file_cyto
        st.session_state["uploaded_file4"] = uploaded_file_nuc

if "uploaded_file3" in st.session_state and "uploaded_file4" in st.session_state:
    uploaded_file_cyto = st.session_state["uploaded_file3"]
    uploaded_file_nuc = st.session_state["uploaded_file4"]
    uploaded_file_cyto.seek(0)
    uploaded_file_nuc.seek(0)
    # Now you can use the uploaded_file as needed

if uploaded_file_cyto is not None and uploaded_file_nuc is not None:
    image_ndarray_cyto = imread(uploaded_file_cyto)
    image_ndarray_nuc = imread(uploaded_file_nuc)
    mergedarrays = np.maximum(image_ndarray_cyto, image_ndarray_nuc)
    st.write("Raw Image (Cyto)")
    image_cyto = st.image(image_ndarray_cyto)
    st.write("Raw Image Nuc")
    image_nuc = st.image(image_ndarray_nuc)

    mask_cellpose = st_run_cellpose(image_ndarray_cyto, model_cellpose)
    mask_stardist = st_run_stardist(
        image_ndarray_nuc, model_stardist, nms_thresh, prob_thresh
    )
    mask_stardist_copy = mask_stardist.copy()

    st.header("Segmentation Results")
    st.subheader("CellPose and Stardist overlayed results")
    fig, ax = plt.subplots(1, 1)
    ax.imshow(mask_cellpose, cmap="viridis")
    lbl_cmap = random_label_cmap()
    ax.imshow(mask_stardist, cmap=lbl_cmap, alpha=0.5)
    ax.axis("off")
    st.pyplot(fig)

    st.header("Full Nucleus Analysis Results")
    df_cellpose = st_df_from_cellpose_mask(mask_cellpose)
    cellpose_df_stat, all_nuc_df_stats = st_predict_all_cells(
        image_ndarray_cyto,
        df_cellpose,
        mask_stardist,
        internalised_threshold=eccentricity_thresh,
    )
    st.dataframe(
        cellpose_df_stat.drop(
            [
                "centroid-0",
                "centroid-1",
                "bbox-0",
                "bbox-1",
                "bbox-2",
                "bbox-3",
                "image",
            ],
            axis=1,
        )
    )
    st.write("Total number of nucleus : ", cellpose_df_stat["N° Nuc"].sum())
    st.write(
        "Total number of internalized nucleus : ",
        cellpose_df_stat["N° Nuc Intern"].sum(),
        " (",
        round(
            100
            * cellpose_df_stat["N° Nuc Intern"].sum()
            / cellpose_df_stat["N° Nuc"].sum(),
            2,
        ),
        "%)",
    )
    st.write(
        "Total number of peripherical nucleus : ",
        cellpose_df_stat["N° Nuc Periph"].sum(),
        " (",
        round(
            100
            * cellpose_df_stat["N° Nuc Periph"].sum()
            / cellpose_df_stat["N° Nuc"].sum(),
            2,
        ),
        "%)",
    )
    st.write(
        "Number of cell with at least one internalized nucleus : ",
        cellpose_df_stat["N° Nuc Intern"].astype(bool).sum(axis=0),
        " (",
        round(
            100
            * cellpose_df_stat["N° Nuc Intern"].astype(bool).sum(axis=0)
            / len(cellpose_df_stat),
            2,
        ),
        "%)",
    )
    st.header("Single Nucleus Analysis Details")
    selected_fiber = st.selectbox("Select a cell", list(range(len(df_cellpose))))
    selected_fiber = int(selected_fiber)
    single_cell_img = mergedarrays[
        df_cellpose.iloc[selected_fiber, 5] : df_cellpose.iloc[selected_fiber, 7],
        df_cellpose.iloc[selected_fiber, 6] : df_cellpose.iloc[selected_fiber, 8],
    ]
    nucleus_single_cell_img = mask_stardist_copy[
        df_cellpose.iloc[selected_fiber, 5] : df_cellpose.iloc[selected_fiber, 7],
        df_cellpose.iloc[selected_fiber, 6] : df_cellpose.iloc[selected_fiber, 8],
    ]
    single_cell_mask = df_cellpose.iloc[selected_fiber, 9]
    single_cell_img[~single_cell_mask] = 0
    nucleus_single_cell_img[~single_cell_mask] = 0

    (
        single_cell_img,
        nucleus_single_cell_img,
        single_cell_mask,
        df_nuc_single,
    ) = st_extract_ROIs(mergedarrays, selected_fiber, df_cellpose, mask_stardist)

    st.markdown(
        """
        * White point represent cell centroid. 
        * Green point represent nucleus centroid. Green dashed line represent the fiber centrer - nucleus distance. 
        * Red point represent the cell border from a straight line between the cell centroid and the nucleus centroid. The red dashed line represent distance between the nucelus and the cell border. 
        * The periphery ratio is calculated by the division of the distance centroid - nucleus and the distance centroid - cell border."""
    )
    fig2, (ax1, ax2) = plt.subplots(1, 2)
    ax1.imshow(single_cell_img, cmap="gray")
    ax2.imshow(nucleus_single_cell_img, cmap="viridis")
    # Plot Fiber centroid
    x_fiber = df_cellpose.iloc[selected_fiber, 3] - df_cellpose.iloc[selected_fiber, 6]
    y_fiber = df_cellpose.iloc[selected_fiber, 2] - df_cellpose.iloc[selected_fiber, 5]

    (
        n_nuc,
        n_nuc_intern,
        n_nuc_periph,
        df_nuc_single_stats,
        ax_nuc,
    ) = st_single_cell_analysis(
        single_cell_img,
        single_cell_mask,
        df_nuc_single,
        x_fiber,
        y_fiber,
        selected_fiber + 1,
        internalised_threshold=eccentricity_thresh,
        draw_and_return=True,
    )

    for index, value in df_nuc_single_stats.iterrows():
        st.write("Nucleus #{} has a periphery ratio of: {}".format(index, value[12]))
    ax1.axis("off")
    ax2.axis("off")
    # st.pyplot(fig2)
    ax_nuc.imshow(single_cell_img)
    ax_nuc.imshow(nucleus_single_cell_img, cmap="viridis", alpha=0.5)
    f = ax_nuc.figure

    st.pyplot(fig2)
    st.pyplot(f)
    st.subheader("All nucleus inside selected cell")

    st.dataframe(
        df_nuc_single_stats.drop(
            [
                "centroid-0",
                "centroid-1",
                "bbox-0",
                "bbox-1",
                "bbox-2",
                "bbox-3",
                "image",
            ],
            axis=1,
        )
    )

    st.header("Painted predicted image")
    st.write(
        "Green color indicates cells with only peripherical nuclei, red color indicates cells with at least one internal nucleus (regeneration)."
    )
    painted_img = st_paint_histo_img(image_ndarray_cyto, df_cellpose, cellpose_df_stat)
    fig3, ax3 = plt.subplots(1, 1)
    cmap = matplotlib.colors.LinearSegmentedColormap.from_list(
        "", ["white", "green", "red"]
    )
    ax3.imshow(mergedarrays)
    ax3.imshow(painted_img, cmap=cmap, alpha=0.5)
    ax3.axis("off")
    st.pyplot(fig3)

html(
    f"""
    <script defer data-domain="lbgi.fr/myoquant" src="https://plausible.cmeyer.fr/js/script.js"></script>
    """
)



import streamlit as st
from streamlit.components.v1 import html

st.write("# MyoQuant-Streamlit🔬")

st.sidebar.success("Select the corresponding staining analysis above.")

st.markdown(
    """
## MyoQuant-Streamlit🔬 is a demo web interface to showcase the usage of [MyoQuant](https://github.com/lambda-science/MyoQuant). 

![MyoQuant Banner](https://i.imgur.com/mzALgZL.png)
![MyoQuant Illustration](https://i.imgur.com/FxpFUT3.png)

[MyoQuant🔬 is a command-line tool to automatically quantify pathological features in muscle fiber histology images.](https://github.com/lambda-science/MyoQuant)  
It is built using CellPose, Stardist, custom neural-network models and image analysis techniques to automatically analyze myopathy histology images. Currently MyoQuant is capable of quantifying centralization of nuclei in muscle fiber with HE staining and anomaly in the mitochondria distribution in muscle fibers with SDH staining.  
This web application is intended for demonstration purposes only.  

## How to Use

Once on the demo, click on the corresponding staining analysis on the sidebar, and upload your histology image. Results will be displayed in the main area automatically.  
For all analysis you can press the "Load Default File" to load a sample image to try the tool.

## Contact
Creator and Maintainer: [**Corentin Meyer**, PhD](https://cmeyer.fr/)  <corentin.meyer@etu.unistra.fr>  
The source code for MyoQuant is available [HERE](https://github.com/lambda-science/MyoQuant).  

## Partners

![Partners Banner](https://i.imgur.com/Xk9wBFQ.png)  
MyoQuant is born within the collaboration between the [CSTB Team @ ICube](https://cstb.icube.unistra.fr/en/index.php/Home) led by Julie D. Thompson, the [Morphological Unit of the Institute of Myology of Paris](https://www.institut-myologie.org/en/recherche-2/neuromuscular-investigation-center/morphological-unit/) led by Teresinha Evangelista, the [imagery platform MyoImage of Center of Research in Myology](https://recherche-myologie.fr/technologies/myoimage/) led by Bruno Cadot, [the photonic microscopy platform of the IGMBC](https://www.igbmc.fr/en/plateformes-technologiques/photonic-microscopy) led by Bertrand Vernay and the [Pathophysiology of neuromuscular diseases team @ IGBMC](https://www.igbmc.fr/en/igbmc/a-propos-de-ligbmc/directory/jocelyn-laporte) led by Jocelyn Laporte.  
"""
)


import streamlit as st
from streamlit.components.v1 import html
import matplotlib
import requests
from io import BytesIO


try:
    from imageio.v2 import imread
except:
    from imageio import imread
import matplotlib.pyplot as plt
import numpy as np
from myoquant.common_func import (
    load_cellpose,
    run_cellpose,
    is_gpu_availiable,
    df_from_cellpose_mask,
)
from myoquant.ATP_analysis import (
    get_all_intensity,
    estimate_threshold,
    plot_density,
    predict_all_cells,
    paint_full_image,
)

labels_predict = {1: "fiber type 1", 2: "fiber type 2"}

use_GPU = is_gpu_availiable()
np.random.seed(42)

@st.cache_resource
def st_load_cellpose():
    return load_cellpose()


@st.cache_data
def st_run_cellpose(image_atp, _model):
    return run_cellpose(image_atp, _model)


@st.cache_data
def st_df_from_cellpose_mask(mask):
    return df_from_cellpose_mask(mask)


@st.cache_data
def st_get_all_intensity(image_atp, df_cellpose):
    return get_all_intensity(image_atp, df_cellpose)


@st.cache_data
def st_estimate_threshold(intensity_list):
    return estimate_threshold(intensity_list)


@st.cache_data
def st_plot_density(all_cell_median_intensity, intensity_threshold):
    return plot_density(all_cell_median_intensity, intensity_threshold)


@st.cache_data
def st_predict_all_cells(image_atp, cellpose_df, intensity_threshold):
    return predict_all_cells(image_atp, cellpose_df, intensity_threshold)


@st.cache_data
def st_paint_full_image(image_atp, df_cellpose, class_predicted_all):
    return paint_full_image(image_atp, df_cellpose, class_predicted_all)


model_cellpose = st_load_cellpose()

with st.sidebar:
    st.write("Threshold Parameters")
    intensity_threshold = st.slider("Intensity Threshold (0=auto)", 0, 255, 0, 5)

st.title("ATP Staining Analysis")
st.write(
    "This demo will automatically quantify the number of type 1 muscle fibers vs the number of type 2 muscle fiber on ATP stained images."
)
default_file_url_5 = "https://raw.githubusercontent.com/lambda-science/MyoQuant/refs/heads/main/sample_img/sample_atp.jpg"


st.write("Upload your ATP Staining image OR click the Load Default File button !")
col1, col2 = st.columns(2)
with col1:
    uploaded_file_atp = st.file_uploader("Choose a file")
    if uploaded_file_atp is not None:
        st.session_state["uploaded_file5"] = uploaded_file_atp

with col2:
    if st.button("Load Default File", type="primary"):
        # Download the default file
        response = requests.get(default_file_url_5)
        # Convert the downloaded content into a file-like object
        uploaded_file_atp = BytesIO(response.content)
        st.session_state["uploaded_file5"] = uploaded_file_atp

if "uploaded_file5" in st.session_state:
    uploaded_file_atp = st.session_state["uploaded_file5"]
    # Now you can use the uploaded_file as needed

if uploaded_file_atp is not None:
    image_ndarray_atp = imread(uploaded_file_atp)

    st.write("Raw Image")
    image = st.image(image_ndarray_atp)

    mask_cellpose = st_run_cellpose(image_ndarray_atp, model_cellpose)

    st.header("Segmentation Results")
    st.subheader("CellPose results")
    fig, ax = plt.subplots(1, 1)
    ax.imshow(mask_cellpose, cmap="viridis")
    ax.axis("off")
    st.pyplot(fig)

    df_cellpose = st_df_from_cellpose_mask(mask_cellpose)

    st.header("Cell Intensity Plot")
    all_cell_median_intensity = st_get_all_intensity(image_ndarray_atp, df_cellpose)
    figure_intensity = st_plot_density(all_cell_median_intensity, intensity_threshold)
    st.pyplot(figure_intensity)

    st.header("ATP Cell Classification Results")
    if intensity_threshold == 0:
        (
            muscle_fiber_type_all,
            all_cell_median_intensity,
            intensity_threshold,
        ) = st_predict_all_cells(
            image_ndarray_atp, df_cellpose, intensity_threshold=None
        )
    else:
        (
            muscle_fiber_type_all,
            all_cell_median_intensity,
            intensity_threshold,
        ) = st_predict_all_cells(
            image_ndarray_atp, df_cellpose, intensity_threshold=intensity_threshold
        )
    df_cellpose["muscle_fiber_type"] = muscle_fiber_type_all
    df_cellpose["median_intensity"] = all_cell_median_intensity
    count_per_label = np.unique(muscle_fiber_type_all, return_counts=True)

    st.dataframe(
        df_cellpose.drop(
            [
                "centroid-0",
                "centroid-1",
                "bbox-0",
                "bbox-1",
                "bbox-2",
                "bbox-3",
                "image",
            ],
            axis=1,
        )
    )
    st.write("Total number of cells detected: ", len(muscle_fiber_type_all))
    for index, elem in enumerate(count_per_label[0]):
        st.write(
            "Number of cells classified as ",
            labels_predict[int(elem)+1],
            ": ",
            count_per_label[1][int(index)],
            " ",
            100 * count_per_label[1][int(index)] / len(muscle_fiber_type_all),
            "%",
        )

    st.header("Painted predicted image")
    st.write(
        "Green color indicates cells classified as control, red color indicates cells classified as sick"
    )
    paint_img = st_paint_full_image(
        image_ndarray_atp, df_cellpose, muscle_fiber_type_all
    )
    fig3, ax3 = plt.subplots(1, 1)
    cmap = matplotlib.colors.LinearSegmentedColormap.from_list(
        "", ["white", "green", "red"]
    )
    ax3.imshow(image_ndarray_atp)
    ax3.imshow(paint_img, cmap=cmap, alpha=0.5)
    ax3.axis("off")
    st.pyplot(fig3)

html(
    f"""
    <script defer data-domain="lbgi.fr/myoquant" src="https://plausible.cmeyer.fr/js/script.js"></script>
    """
)



from huggingface_hub import HfApi
api = HfApi()
api.upload_folder(
    folder_path="src/myoquant/streamlit",
    repo_id="corentinm7/MyoQuant",
    repo_type="space",
    ignore_patterns=["*.h5", "*.keras"]
)


import streamlit as st
from streamlit.components.v1 import html

st.set_page_config(
    page_title="MyoQuant",
    page_icon="🔬",
)

navigation_dict = {
    "MyoQuant": [
        st.Page(
            page="pages/home.py",
            title="Home",
            icon="🏠",
            default=True,
        ),
        st.Page(
            page="pages/ATP_staining.py",
            title="ATP Staining",
            icon="🟣",
            default=False,
        ),
        st.Page(
            page="pages/HE_staining.py",
            title="HE Staining",
            icon="🔴",
            default=False,
        ),
        st.Page(
            page="pages/SDH_staining.py",
            title="SDH Staining",
            icon="🔵",
            default=False,
        ),
        st.Page(
            page="pages/Fluo_staining.py",
            title="Fluo Staining",
            icon="🟢",
            default=False,
        ),
    ],
}
selected_page = st.navigation(navigation_dict, position="sidebar")
selected_page.run()



import streamlit as st
from streamlit.components.v1 import html
import requests
from io import BytesIO

try:
    from imageio.v2 import imread
except:
    from imageio import imread
import pandas as pd
import matplotlib
import matplotlib.pyplot as plt
import tensorflow as tf
import numpy as np
from os import path
import urllib.request
from myoquant.SDH_analysis import (
    predict_all_cells,
    predict_single_cell,
    paint_full_image,
)
from myoquant.common_func import (
    load_cellpose,
    run_cellpose,
    is_gpu_availiable,
    df_from_cellpose_mask,
    load_sdh_model,
    extract_single_image,
)


use_GPU = is_gpu_availiable()


@st.cache_resource
def st_load_sdh_model():
    return load_sdh_model()


@st.cache_resource
def st_load_cellpose():
    return load_cellpose()


@st.cache_data
def st_run_cellpose(image_ndarray, _model):
    return run_cellpose(image_ndarray, _model)


@st.cache_data
def st_df_from_cellpose_mask(mask):
    return df_from_cellpose_mask(mask)


@st.cache_data
def st_predict_all_cells(image_ndarray, cellpose_df, _model_SDH):
    return predict_all_cells(image_ndarray, cellpose_df, _model_SDH)


@st.cache_data
def st_extract_single_image(image_ndarray, cellpose_df, index):
    return extract_single_image(image_ndarray, cellpose_df, index)


@st.cache_data
def st_predict_single_cell(image_ndarray, _model_SDH):
    return predict_single_cell(image_ndarray, _model_SDH)


@st.cache_data
def st_paint_full_image(image_sdh, df_cellpose, class_predicted_all):
    return paint_full_image(image_sdh, df_cellpose, class_predicted_all)


labels_predict = ["control", "sick"]

tf.random.set_seed(42)
np.random.seed(42)



with st.spinner("Please wait we are loading the SDH Model."):
    model_SDH = st_load_sdh_model()

st.success("SDH Model have been downloaded !")

model_cellpose = st_load_cellpose()

st.title("SDH Staining Analysis")
st.write(
    "This demo will automatically detect cells classify the SDH stained cell as sick or healthy using our deep-learning model."
)


default_file_url_2 = "https://raw.githubusercontent.com/lambda-science/MyoQuant/refs/heads/main/sample_img/sample_sdh.jpg"


st.write("Upload your SDH Staining image OR click the Load Default File button !")
col1, col2 = st.columns(2)
with col1:
    uploaded_file_sdh = st.file_uploader("Choose a file")
    if uploaded_file_sdh is not None:
        st.session_state["uploaded_file2"] = uploaded_file_sdh

with col2:
    if st.button("Load Default File", type="primary"):
        # Download the default file
        response = requests.get(default_file_url_2)
        # Convert the downloaded content into a file-like object
        uploaded_file_sdh = BytesIO(response.content)
        st.session_state["uploaded_file2"] = uploaded_file_sdh

if "uploaded_file2" in st.session_state:
    uploaded_file_sdh = st.session_state["uploaded_file2"]
    # Now you can use the uploaded_file as needed
    uploaded_file_sdh.seek(0)  # reset the file pointer to the start


if uploaded_file_sdh is not None:
    image_ndarray_sdh = imread(uploaded_file_sdh)

    st.write("Raw Image")
    image = st.image(image_ndarray_sdh)

    mask_cellpose = st_run_cellpose(image_ndarray_sdh, model_cellpose)

    st.header("Segmentation Results")
    st.subheader("CellPose results")
    fig, ax = plt.subplots(1, 1)
    ax.imshow(mask_cellpose, cmap="viridis")
    ax.axis("off")
    st.pyplot(fig)

    st.header("SDH Cell Classification Results")
    df_cellpose = st_df_from_cellpose_mask(mask_cellpose)
    df_cellpose_results = st_predict_all_cells(
        image_ndarray_sdh, df_cellpose, model_SDH
    )
    class_predicted_all = df_cellpose_results["class_predicted"].values
    proba_predicted_all = df_cellpose_results["proba_predicted"].values
    count_per_label = np.unique(class_predicted_all, return_counts=True)
    class_and_proba_df = pd.DataFrame(
        list(zip(class_predicted_all, proba_predicted_all)),
        columns=["class", "proba"],
    )
    st.dataframe(
        df_cellpose_results.drop(
            [
                "centroid-0",
                "centroid-1",
                "bbox-0",
                "bbox-1",
                "bbox-2",
                "bbox-3",
                "image",
            ],
            axis=1,
        )
    )
    st.write("Total number of cells detected: ", len(class_predicted_all))
    for elem in count_per_label[0]:
        st.write(
            "Number of cells classified as ",
            labels_predict[int(elem)],
            ": ",
            count_per_label[1][int(elem)],
            " ",
            100 * count_per_label[1][int(elem)] / len(class_predicted_all),
            "%",
        )

    st.header("Single Cell Grad-CAM")
    selected_fiber = st.selectbox("Select a cell", list(range(len(df_cellpose))))
    selected_fiber = int(selected_fiber)
    single_cell_img = st_extract_single_image(
        image_ndarray_sdh, df_cellpose, selected_fiber
    )

    grad_img, class_predicted, proba_predicted = st_predict_single_cell(
        single_cell_img, model_SDH
    )

    fig2, (ax1, ax2) = plt.subplots(1, 2)
    resized_single_cell_img = tf.image.resize(single_cell_img, (256, 256))
    ax1.imshow(single_cell_img)
    ax2.imshow(grad_img)
    ax1.axis("off")
    # ax2.axis("off")

    xlabel = (
        labels_predict[int(class_predicted)]
        + " ("
        + str(round(proba_predicted, 2))
        + ")"
    )
    ax2.set_xlabel(xlabel)
    st.pyplot(fig2)

    st.header("Painted predicted image")
    st.write(
        "Green color indicates cells classified as control, red color indicates cells classified as sick"
    )
    paint_img = st_paint_full_image(image_ndarray_sdh, df_cellpose, class_predicted_all)
    fig3, ax3 = plt.subplots(1, 1)
    cmap = matplotlib.colors.LinearSegmentedColormap.from_list(
        "", ["white", "green", "red"]
    )
    ax3.imshow(image_ndarray_sdh)
    ax3.imshow(paint_img, cmap=cmap, alpha=0.5)
    ax3.axis("off")
    st.pyplot(fig3)

html(
    f"""
    <script defer data-domain="lbgi.fr/myoquant" src="https://plausible.cmeyer.fr/js/script.js"></script>
    """
)



from typer.testing import CliRunner

from myoquant.__main__ import app

runner = CliRunner()


def test_sdh_analysis():
    result = runner.invoke(
        app,
        [
            "sdh-analysis",
            "sample_img/sample_sdh.jpg",
            "--cellpose-diameter",
            80,
            "--mask-path",
            "sample_img/binary_mask_sdh.tif",
        ],
    )
    assert result.exit_code == 0
    assert "Analysis Results" in result.stdout


def test_he_analysis():
    result = runner.invoke(
        app,
        [
            "he-analysis",
            "sample_img/sample_he.jpg",
            "--cellpose-diameter",
            80,
            "--mask-path",
            "sample_img/binary_mask.tif",
        ],
    )
    assert result.exit_code == 0
    assert "Analysis Results" in result.stdout


def test_he_analysis_fluo():
    result = runner.invoke(
        app,
        [
            "he-analysis",
            "sample_img/cytoplasm.tif",
            "--cellpose-diameter",
            80,
            "--mask-path",
            "sample_img/binary_mask_fluo.tif",
            "--fluo-nuc",
            "sample_img/nuclei.tif",
        ],
    )
    assert result.exit_code == 0
    assert "Analysis Results" in result.stdout


def test_atp_analysis():
    result = runner.invoke(
        app,
        [
            "atp-analysis",
            "sample_img/sample_atp.jpg",
            "--cellpose-diameter",
            80,
        ],
    )
    assert result.exit_code == 0
    assert "Analysis Results" in result.stdout



import typer
from rich.console import Console
import pkg_resources

__version__ = pkg_resources.get_distribution("myoquant").version
__version_cellpose__ = pkg_resources.get_distribution("cellpose").version
__version_stardist__ = pkg_resources.get_distribution("stardist").version
__version_torch__ = pkg_resources.get_distribution("torch").version
__version_tensorflow__ = pkg_resources.get_distribution("tensorflow").version

from myoquant.commands.docs import app as docs_app
from myoquant.commands import run_sdh, run_he, run_atp

console = Console()


def version_callback(value: bool):
    if value:
        print(
            f"MyoQuant Version: {__version__} \nCellpose Version: {__version_cellpose__} \nStardist Version: {__version_stardist__} \nTorch Version: {__version_torch__} \nTensorflow Version: {__version_tensorflow__}"
        )
        raise typer.Exit()


app = typer.Typer(
    name="MyoQuant",
    add_completion=False,
    help="MyoQuant Analysis Command Line Interface",
    pretty_exceptions_show_locals=False,
)
app.add_typer(docs_app, name="docs", help="Generate documentation")

app.registered_commands += (
        run_sdh.app.registered_commands
        + run_he.app.registered_commands
        + run_atp.app.registered_commands
)

@app.callback()
def main(
    version: bool = typer.Option(
        None, "--version", callback=version_callback, is_eager=True
    )
):
    """
    MyoQuant Analysis Command Line Interface
    """

if __name__ == "__main__":
    app()



import pandas as pd
from scipy.stats import gaussian_kde
from sklearn.mixture import GaussianMixture
from myoquant.common_func import extract_single_image, df_from_cellpose_mask
import numpy as np
import matplotlib

matplotlib.use("agg")

import matplotlib.pyplot as plt

labels_predict = {1: "fiber type 1", 2: "fiber type 2"}
np.random.seed(42)


def get_all_intensity(
    image_array, df_cellpose, intensity_method="median", erosion=None
):
    all_cell_median_intensity = []
    for index in range(len(df_cellpose)):
        single_cell_img = extract_single_image(image_array, df_cellpose, index, erosion)

        # Calculate median pixel intensity of the cell but ignore 0 values
        if intensity_method == "median":
            single_cell_median_intensity = np.median(
                single_cell_img[single_cell_img > 0]
            )
        elif intensity_method == "mean":
            single_cell_median_intensity = np.mean(single_cell_img[single_cell_img > 0])
        all_cell_median_intensity.append(single_cell_median_intensity)
    return all_cell_median_intensity


def estimate_threshold(intensity_list, n_classes=2):
    density = gaussian_kde(intensity_list)
    density.covariance_factor = lambda: 0.05
    density._compute_covariance()

    # Create a vector of 256 values going from 0 to 256:
    xs = np.linspace(0, 255, 256)
    density_xs_values = density(xs)
    gmm = GaussianMixture(n_components=n_classes).fit(
        np.array(intensity_list).reshape(-1, 1)
    )

    # Find the x values of the two peaks
    peaks_x = np.sort(gmm.means_.flatten())
    # Find the minimum point between the two peaks

    threshold_list = []
    length = len(peaks_x)
    for index, peaks in enumerate(peaks_x):
        if index == length - 1:
            break
        min_index = np.argmin(
            density_xs_values[(xs > peaks) & (xs < peaks_x[index + 1])]
        )
        threshold_list.append(peaks + xs[min_index])
    return threshold_list


def plot_density(all_cell_median_intensity, intensity_threshold, n_classes=2):
    if intensity_threshold == 0:
        intensity_threshold = estimate_threshold(all_cell_median_intensity, n_classes)
    fig, ax = plt.subplots(figsize=(10, 5))
    density = gaussian_kde(all_cell_median_intensity)
    density.covariance_factor = lambda: 0.1
    density._compute_covariance()

    # Create a vector of 256 values going from 0 to 25
    xs = np.linspace(0, 255, 256)
    density_xs_values = density(xs)
    ax.hist(
        all_cell_median_intensity, bins=255, density=True, alpha=0.5, label="Histogram"
    )
    ax.plot(xs, density_xs_values, label="Estimated Density", linewidth=3)
    if isinstance(intensity_threshold, int):
        intensity_threshold = [intensity_threshold]
    for values in intensity_threshold:
        ax.axvline(x=values, color="red", label="Threshold")
    ax.set_xlabel("Pixel Intensity")
    ax.set_ylabel("Density")
    ax.legend()
    return fig


def merge_peaks_too_close(peak_list):
    pass


def classify_cells_intensity(all_cell_median_intensity, intensity_threshold):
    muscle_fiber_type_all = []
    for intensity in all_cell_median_intensity:
        if isinstance(intensity_threshold, int):
            intensity_threshold = [intensity_threshold]
        class_cell = np.searchsorted(intensity_threshold, intensity, side="right")
        muscle_fiber_type_all.append(class_cell)
    return muscle_fiber_type_all


def predict_all_cells(
    histo_img,
    cellpose_df,
    intensity_threshold,
    n_classes=2,
    intensity_method="median",
    erosion=None,
):
    all_cell_median_intensity = get_all_intensity(
        histo_img, cellpose_df, intensity_method, erosion
    )
    if intensity_threshold is None:
        intensity_threshold = estimate_threshold(all_cell_median_intensity, n_classes)

    muscle_fiber_type_all = classify_cells_intensity(
        all_cell_median_intensity, intensity_threshold
    )
    return muscle_fiber_type_all, all_cell_median_intensity, intensity_threshold


def paint_full_image(image_atp, df_cellpose, class_predicted_all):
    image_atp_paint = np.zeros(
        (image_atp.shape[0], image_atp.shape[1]), dtype=np.uint16
    )
    # for index in track(range(len(df_cellpose)), description="Painting cells"):
    for index in range(len(df_cellpose)):
        single_cell_mask = df_cellpose.iloc[index, 9].copy()
        image_atp_paint[
            df_cellpose.iloc[index, 5] : df_cellpose.iloc[index, 7],
            df_cellpose.iloc[index, 6] : df_cellpose.iloc[index, 8],
        ][single_cell_mask] = (
            class_predicted_all[index] + 1
        )
    return image_atp_paint


def label_list_from_threhsold(threshold_list):
    label_list = []
    length = len(threshold_list)
    for index, threshold in enumerate(threshold_list):
        if index == 0:
            label_list.append(f"<{threshold}")
        if index == length - 1:
            label_list.append(f">{threshold}")
        else:
            label_list.append(f">{threshold} & <{threshold_list[index+1]}")
    return label_list


def run_atp_analysis(
    image_array,
    mask_cellpose,
    intensity_threshold=None,
    n_classes=2,
    intensity_method="median",
    erosion=None,
):
    df_cellpose = df_from_cellpose_mask(mask_cellpose)
    class_predicted_all, intensity_all, intensity_threshold = predict_all_cells(
        image_array,
        df_cellpose,
        intensity_threshold,
        n_classes,
        intensity_method,
        erosion,
    )
    fig = plot_density(intensity_all, intensity_threshold, n_classes)
    df_cellpose["muscle_cell_type"] = class_predicted_all
    df_cellpose["cell_intensity"] = intensity_all
    count_per_label = np.unique(class_predicted_all, return_counts=True)

    # Result table dict
    headers = ["Feature", "Raw Count", "Proportion (%)"]
    data = []
    data.append(["Muscle Fibers", len(class_predicted_all), 100])
    label_list = label_list_from_threhsold(intensity_threshold)
    for index, elem in enumerate(count_per_label[0]):
        data.append(
            [
                label_list[int(elem)],
                count_per_label[1][int(index)],
                100 * count_per_label[1][int(index)] / len(class_predicted_all),
            ]
        )
    result_df = pd.DataFrame(columns=headers, data=data)
    # Paint The Full Image
    full_label_map = paint_full_image(image_array, df_cellpose, class_predicted_all)
    return result_df, full_label_map, df_cellpose, fig
